/*
 * slashexec - A CLI for gaim
 * Copyright (C) 2004 Gary Kramlich
 * Copyright (C) 2005 Peter Lawler
 * Copyright (C) 2005 Daniel Atallah
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#if defined HAVE_CONFIG_H && !defined _WIN32
# include "../se_config.h"
#endif

#include <glib.h>
#include <string.h>
#ifndef _WIN32
# include <stdio.h>
#endif

#ifdef USE_TILDE
# include <readline/tilde.h>
# include <readline/readline.h>
#endif

#define GAIM_PLUGINS

#include "cmds.h"
#include "conversation.h"
#include "debug.h"
#include "notify.h"
#include "plugin.h"
#include "util.h"
#include "version.h"

/* I might convert these to preferences,
 * but my pref panel already exceeds 1600 wide in head
 */
#define MAX_COMMAND_LEN 2048

static GaimCmdId se_cmd;

static gchar *
se_spawn(const gchar *cmd) {
	gchar buffer[MAX_COMMAND_LEN], **argv;
	GError *error;
	gint argc;
	error = NULL;
	/* TODO: need to properly escape cmd at some stage*/
	gaim_debug_info("slashexec", "Executing: %s\n", cmd);
	g_snprintf (buffer, MAX_COMMAND_LEN, "%s", cmd);
	if (!g_shell_parse_argv (buffer, &argc, &argv, &error)) {
		if (buffer)
			gaim_debug_info("slashexec", "Unable to parse \"%s\"\n", buffer);
		if (error) {
			gaim_debug_info("slashexec", "Parse error message: %s\n",
							error->message);
			g_error_free(error);
		}
		g_strfreev(argv);
		return NULL;
	}
	gchar *standard_output = NULL, *standard_error = NULL;
	GSpawnFlags Gflags;
	Gflags = G_SPAWN_SEARCH_PATH;
	if (!g_spawn_sync (NULL, argv, NULL, Gflags,
		NULL, NULL, &standard_output, &standard_error, NULL, &error)) {
		if (buffer)
			gaim_debug_info("slashexec", "Unable to exec: \"%s\"\n", buffer);
		if (error) {
			/* TODO: need to pass the error somewhere so it turns up in the convo*/
			gaim_debug_info("slashexec", "Error message: %s\n",
							error->message);
			/* for example....
			 *if (error->code == G_FILE_ERROR_NOENT) {
			 *	*errorcode = g_strdup(error->message);
			 * }
			*/
			g_error_free (error);
		}
		g_free(standard_output);
		g_free(standard_error);
		g_strfreev(argv);
		return NULL;
	}
	if (error)
		g_error_free (error);
	g_strfreev(argv);
	g_free(standard_error);
	g_strchug(standard_output);
	return standard_output;
}

static GaimCmdRet
se_cmd_cb(GaimConversation *c, const gchar *cmd, gchar **args, gchar **error,
			gpointer data) {
	gchar *cmdl, *output, *error_string, *tmp;
	gboolean send = FALSE;
	if(args[0] == NULL)
		return GAIM_CMD_RET_FAILED;
	if(!args[0])
		return GAIM_CMD_RET_FAILED;
	if(!strncmp(args[0], "-o", 2)) {
		send = TRUE;
		cmdl = g_strdup_printf("%s", args[0] + 3);
	} else {
		cmdl = g_strdup_printf("%s", args[0]);
	}
	tmp = cmdl;
#ifndef _WIN32
	cmdl = tilde_expand(tmp);
	g_free(tmp);
#else /* on WIN32 convert '\' to '/' */
	cmdl = g_strdelimit(tmp, "\\", '/');
#endif
	if (strlen(cmdl) > MAX_COMMAND_LEN)
		return GAIM_CMD_RET_FAILED;
	output = se_spawn(cmdl);
	if(output) {
		if (g_str_has_suffix(output, "\n"))
			output[strlen(output) - 1] = '\0';
		if(send) {
			GaimConversationType type;
			type = gaim_conversation_get_type(c);
			if(type == GAIM_CONV_IM) {
				gaim_conv_im_send(GAIM_CONV_IM(c), output);
			} else if(type == GAIM_CONV_CHAT) {
				gaim_conv_chat_send(GAIM_CONV_CHAT(c), output);
			}
		} else {
			gaim_conversation_write(c, NULL, output, GAIM_MESSAGE_SYSTEM,
									time(NULL));
		}
	} else {
		/* TODO: Convert to proper error messages */
		error_string = g_strdup_printf("Error executing: %s", cmdl);
		gaim_debug_info("slashexec","%s\n", error_string);
		gaim_debug_info("slashexec","cmdstatus %s\n", error);
		gaim_conversation_write(c, NULL, error_string, GAIM_MESSAGE_SYSTEM,
								time(NULL));
		g_free(error_string);
		g_free(cmdl);
		g_free(output);
		return GAIM_CMD_RET_FAILED;
	}
	g_free(cmdl);
	g_free(output);
	return GAIM_CMD_RET_OK;
}

static gboolean
se_load(GaimPlugin *plugin) {
	const gchar *help = "exec [-o] &lt;command&gt;, runs the command.\n"
						"If -o flag is used then output is sent to current channel, "
						"else is printed to current text box";
	se_cmd = gaim_cmd_register("exec", "s", GAIM_CMD_P_PLUGIN,
								GAIM_CMD_FLAG_IM | GAIM_CMD_FLAG_CHAT, NULL,
								se_cmd_cb, help, NULL);
#if GAIM_VERSION_CHECK(2,0,0)
	gaim_debug_register_category("slashexec");
#endif
	return TRUE;
}

static gboolean
se_unload(GaimPlugin *plugin) {
	gaim_cmd_unregister(se_cmd);
#if 0
/* gaim_debug_unregister_category doesn't seem to be available yet*/
#if GAIM_VERSION_CHECK(2,0,0)
	gaim_debug_unregister_category("slashexec");
#endif
#endif
	return TRUE;
}

static GaimPluginInfo se_info = {
	GAIM_PLUGIN_MAGIC,
	GAIM_MAJOR_VERSION,
	GAIM_MINOR_VERSION,
	GAIM_PLUGIN_STANDARD,								/* type				*/
	NULL,												/* ui requirement	*/
	0,													/* flags			*/
	NULL,												/* dependencies		*/
	GAIM_PRIORITY_DEFAULT,								/* priority			*/
	"core-amc_grim-slashexec",							/* id				*/
	"/exec",											/* name				*/
	SE_VERSION,											/* version			*/
	"/exec ah la unix IRC CLI",							/* summary			*/
	"A plugin that adds the /exec command\n"
	"line interpreter like most unix/linux\n"
	"irc clients.\n",									/* description		*/
	"Gary Kramlich <amc_grim@users.sf.net>\n"
	"Peter Lawler <bleeter from users.sf.net>",		/* author			*/
	"http://guifications.sourceforge.net/slashexec/",	/* homepage			*/
	se_load,											/* load				*/
	se_unload,											/* unload			*/
	NULL,												/* destroy			*/
	NULL,												/* ui info			*/
	NULL,												/* extra info		*/
	NULL,												/* actions info		*/
	NULL
};

static void
init_plugin(GaimPlugin *plugin) {
}

GAIM_INIT_PLUGIN(slashexec, init_plugin, se_info)
